@@ -28,21 +28,24 @@ class Activity(object):
     >>> a = vipy.object.Activity(startframe=0, endframe=10, category='Walking', tracks={t.id():t})
 
     """
-    def __init__(self, startframe, endframe, framerate=None, label=None, shortlabel=None, category=None, tracks=None, attributes=None, actorid=None, confidence=None):
+    def __init__(self, startframe, endframe, framerate=None, label=None, shortlabel=None, category=None, tracks=None, attributes=None, actorid=None, confidence=None, id=None):
         assert not (label is not None and category is not None), "Activity() Constructor requires either label or category kwargs, not both"
         assert startframe <= endframe, "Start frame must be less than or equal to end frame"
         if tracks:
             tracks = [v for (k,v) in tracks.items()] if isinstance(tracks, dict) else tracks  # backwards compatible, track dictionary input
             assert (all([isstring(t) for t in tracks]) or all([isinstance(t, Track) for t in tracks])), "Invalid track input"
             if all([isinstance(t, Track) for t in tracks]):
-                assert all([any([t.during(f) for f in range(startframe, endframe)]) for t in tracks]), "All tracks must be be present in at least one frame when this activity occurs"
+                assert any([any([t.during(f) for f in range(startframe, endframe)]) for t in tracks]), "At least one track must be be present in at least one frame when this activity occurs"
                 tracks = [t.id() for t in tracks]  
         trackid = set(tracks) if tracks is not None else (set([actorid]) if actorid is not None else set([]))  # only store IDs, not tracks
         
         if tracks is not None and actorid is not None and actorid not in trackid:
             trackid.add(actorid)
 
-        global ACTIVITY_GUID; self._id = hex(int(ACTIVITY_GUID))[2:];  ACTIVITY_GUID = ACTIVITY_GUID + 1;  # faster, increment package level UUID4 initialized GUID
+        if id is None:
+            global ACTIVITY_GUID; self._id = hex(int(ACTIVITY_GUID))[2:];  ACTIVITY_GUID = ACTIVITY_GUID + 1;  # faster, increment package level UUID4 initialized GUID
+        else:
+            self._id = id  # use provided
         self._startframe = int(startframe)
         self._endframe = int(endframe)
         self._framerate = framerate
@@ -75,12 +78,31 @@ class Activity(object):
                    shortlabel=d['_shortlabel'],
                    tracks=d['_trackid'],
                    attributes=d['attributes'],
-                   actorid=d['_actorid'])
+                   actorid=d['_actorid'],
+                   id=d['_id'] if '_id' in d else None)
                 
     def __len__(self):
         """Return activity length in frames, or zero if degenerate"""
         return max(0, self.endframe() - self.startframe())
 
+    def duration(self, s=None, centered=False):
+        """The length of the activity in seconds.
+
+        Args:
+            s: [float] The number of seconds for this activity, starting at the startframe
+            centered: [bool] If true, then set the duration centered on the middle frame
+
+        Returns:
+            The duration in seconds of this activity object (if s=None)
+            This activity object with the requested duration (if s!=None)
+        """
+        assert self.framerate() is not None, "Framerate must be set in constructor"
+        if s is None:
+            return len(self) / float(self.framerate())
+        else:
+            return (self.endframe(self.startframe() + int(round(s*self.framerate()))) if not centered else 
+                    self.truncate(startframe=self.middleframe()-int(np.ceil(s*self.framerate())//2), endframe=self.middleframe()+int(np.ceil(s*self.framerate())//2)))
+        
     def __repr__(self):
         return str('<vipy.activity: category="%s", frames=(%d,%d), tracks=%s%s>' % (self.category(), self.startframe(), self.endframe(), len(self.trackids()), '' if self.confidence() is None else ', confidence=%1.2f' % self.confidence()))
 
@@ -97,6 +119,8 @@ class Activity(object):
             return self._actorid
         else:
             self._actorid = actorid
+            if actorid not in self._trackid:
+                self._trackid.add(actorid) if isinstance(self._trackid, set) else self._trackid.append(actorid)                    
             return self
 
     def startframe(self, f=None):
@@ -116,6 +140,11 @@ class Activity(object):
     def middleframe(self):
         return int(np.round((self._endframe - self._startframe) / 2.0)) + self._startframe
 
+    def _set_framerate(self, fps):
+        """Override framerate conversion and just set the framerate attribute.  This should really be set only in constructor.  Use with caution!"""
+        self._framerate = fps
+        return self
+
     def framerate(self, fps=None, speed=None):
         """Resample (startframe, endframe) from known original framerate set by constructor to be new framerate fps"""        
         if fps is None and speed is None:
@@ -132,16 +161,17 @@ class Activity(object):
             self._framerate = fps
             return self
     
-    def category(self, label=None):
+    def category(self, label=None, shortlabel=None):
+        """Change the label (and shortlabel) to the new label (and shortlabel)"""
         if label is not None:
             self._label = label
-            return self
+            return self.shortlabel(shortlabel) if shortlabel is not None else self
         else:
             return self._label
 
-    def label(self, label=None):
+    def label(self, label=None, shortlabel=None):
         """Alias for category"""
-        return self.category(label)
+        return self.category(label, shortlabel=shortlabel)
 
     def shortlabel(self, label=None):
         """A optional shorter label string to show in the visualizations"""                
@@ -154,14 +184,21 @@ class Activity(object):
     def add(self, track):
         """Add the track id for the track to this activity, so that if the track is changed externally it is reflected here"""
         assert isinstance(track, Track), "Invalid input - must be vipy.object.Track"
-        assert track.during(self.startframe()) or track.during(self.endframe()) or track.during(self.middleframe()), "The track must be present during the activity"
-        self._trackid.add(track.id())
+        assert self.during_interval(track.startframe(), track.endframe()), "The track must be present during the activity"
+        if track.id() not in self._trackid:
+            self._trackid = list(self._trackid)
+            self._trackid.append(track.id())
         return self
         
     def tracks(self):
         """alias for trackids"""
         return self.trackids()
 
+    def cleartracks(self):
+        """Remove all track IDs from this activity"""
+        self._trackid = []
+        return self
+
     def trackids(self):
         """Return a set of track IDs associated with this activity"""
         return set(self._trackid)
@@ -181,17 +218,23 @@ class Activity(object):
         trackid = track.id() if isinstance(track, Track) else track
         return trackid in self._trackid
 
+    def hastrackoverlap(self, track):
+        """is the activity occurring during the interval when the track is occurring?"""
+        assert isinstance(track, Track)
+        return self.hastrack(track) and self.temporal_iou(track) > 0
+    
     def append(self, newtrack):
         """Append newtrack to this activity and set as actorid()"""
         assert isinstance(newtrack, Track), "Invalid input - must be vipy.object.Track"
-        self._trackid.add(newtrack.id())
+        self._trackid.add(newtrack.id()) if isinstance(self._trackid, set) else self._trackid.append(newtrack.id())
         self.actorid(newtrack.id())
         return self
 
     def trackfilter(self, f):
         """Remove all tracks such that the lambda function f(trackid) resolves to False"""
+        assert callable(f)
         self._trackid = [tid for tid in self._trackid if f(tid)]
-        if not f(self.actorid()):
+        if self.actorid() not in self._trackid:
             self._actorid = None
         return self
 
@@ -208,10 +251,12 @@ class Activity(object):
     def replaceid(self, oldtrackid, newtrackid):
         """Replace oldtrack with newtrack if present in self._tracks.  Pass in a trackdict to share reference to track, so that track owner can modify the track and this object observes the change"""
         if self.hastrack(oldtrackid):
-            self._trackid.discard(oldtrackid)
-            self._trackid.add(newtrackid)
+            ti = set(self._trackid)
+            ti.discard(oldtrackid)
+            ti.add(newtrackid)
             if self.actorid() == oldtrackid:
                 self.actorid(newtrackid)
+            self._trackid = list(ti)
         return self    
     
     def during(self, frame):
@@ -223,6 +268,7 @@ class Activity(object):
         ef = endframe+(0 if not inclusive else 1)
         return (startframe >= self._startframe and startframe <= self._endframe) or (ef >= self._startframe and ef <= self._endframe) or (startframe <= self._startframe and ef >= self._endframe)
 
+    
     def union(self, other, confweight=0.5, maxconf=False):
         """Compute the union of the new activity other to this activity by updating the start and end times and computing the mean confidence.
         
@@ -242,15 +288,17 @@ class Activity(object):
         return self
         
     def temporal_iou(self, other):
-        """Return the temporal intersection over union of two activities"""
-        assert isinstance(other, Activity), "Invalid input - must be vipy.object.Activity()"
-        assert self._framerate == other._framerate, "invald input - framerate must match"
-        t_start = min(self._startframe, other._startframe)
-        t_end = max(self._endframe, other._endframe)
+        """Return the temporal intersection over union of two activities or this activity and a track"""
+        assert isinstance(other, Activity) or isinstance(other, Track), "Invalid input - must be vipy.activity.Activity or vipy.object.Track"
+        assert self._framerate == other._framerate, "invalid input - framerate must match"
+        (sf, ef) = (other._startframe, other._endframe) if isinstance(other, Activity) else (other.startframe(), other.endframe())  # attribute access is faster than methods
+        
+        t_start = min(self._startframe, sf)
+        t_end = max(self._endframe, ef)
         t_union = float(t_end - t_start)
         
-        t_start = max(self._startframe, other._startframe)
-        t_end = min(self._endframe, other._endframe)
+        t_start = max(self._startframe, sf)
+        t_end = min(self._endframe, ef)
         t_intersection = float(t_end - t_start)
         
         return (t_intersection / t_union) if t_intersection > 0 else 0
@@ -264,6 +312,7 @@ class Activity(object):
         """Truncate the activity so that it is between startframe and endframe"""
         self._startframe = self._startframe if startframe is None else max(self._startframe, startframe)
         self._endframe = self._endframe if endframe is None else min(self._endframe, endframe)
+        self._endframe = self._endframe if self._endframe > self._startframe else self._startframe  # degenerate truncation
         return self
             
     def id(self, newid=None):
@@ -281,7 +330,7 @@ class Activity(object):
         return a
     
     def temporalpad(self, df):
-        """Add a temporal pad of df=(before,after) or df=pad frames to the start and end of the activity.  The padded start frame may be negative."""
+        """Add a temporal pad of df=(before frames, after frames) or df=pad frames to the start and end of the activity.  The padded start frame may be negative."""
         df = (df, df) if not isinstance(df, tuple) else df
         self._startframe -= int(df[0])
         self._endframe += int(df[1])
@@ -289,7 +338,7 @@ class Activity(object):
 
     def padto(self, t):
         """Add a symmetric temporal pad so that the activity is at least t seconds long"""
-        return self.temporalpad((t - len(self))/2.0) if t > len(self) else self
+        return self.temporalpad(int(np.ceil(self.framerate()*((t - self.duration())/2.0)))) if t > self.duration() else self
 
     def disjoint(self, other, strict=False):
         """Enforce disjoint activities with other by shifting the endframe or startframe of self to not overlap if they share the same tracks.
@@ -311,3 +360,8 @@ class Activity(object):
         """Return the temporal distance in frames between self and other which is the minimum frame difference between the end of one to the start of the other, or zero if they overlap"""
         assert isinstance(other, Activity), "Invalid input - must be vipy.activity.Activity()"
         return (max(self.startframe(), other.startframe()) - min(self.endframe(), other.endframe())) if self.temporal_iou(other) == 0 else 0
+
+    def within(self, startframe, endframe):
+        """Is the activity within the frame rate (startframe, endframe)?"""
+        return self.startframe() >= startframe and self.endframe() <= endframe
+    
