@@ -32,6 +32,22 @@ class Dask(object):
 
         self._num_processes = num_processes
 
+        os.environ['DASK_LOGGING__DISTRIBUTED'] = 'warning' if not verbose else 'info'
+        env = {'VIPY_BACKEND':'Agg',  # headless 
+               'PYTHONOPATH':os.environ['PYTHONPATH'] if 'PYTHONPATH' in os.environ else '',
+               'PATH':os.environ['PATH'] if 'PATH' in os.environ else '',
+               #'DASK_DISTRIBUTED__COMM__TIMEOUTS__TCP':"30s",
+               'DASK_DISTRIBUTED__COMM__TIMEOUTS__CONNECT':"30s",
+               #'DASK_DISTRIBUTED__SCHEDULER__WORK_STEALING':'false',
+               #'DASK_DISTRIBUTED__ADMIN__TICK__LIMIT':"30s",
+               #'DASK_DISTRIBUTED__ADMIN__TICK__INTERVAL':"2s",
+               'DASK_DISTRIBUTED__DEPLOY__LOST_WORKER_TIMEOUT':"30s"}
+        
+        dask.config.set({'DISTRIBUTED.COMM.TIMEOUTS.CONNECT'.lower():'30s'})
+        dask.config.set({'DISTRIBUTED.COMM.TIMEOUTS.TCP'.lower():'30s'})
+        dask.config.set({'DISTRIBUTED.DEPLOY.LOST_WORKER_TIMEOUT'.lower():'30s'})
+        dask.config.refresh()
+
         if address is not None:
             # Distributed scheduler
             self._client = Client(name='vipy', address=address)
@@ -44,18 +60,10 @@ class Dask(object):
                                   processes=True, 
                                   threads_per_worker=1,
                                   n_workers=num_processes, 
-                                  env={'VIPY_BACKEND':'Agg',  # headless 
-                                       'PYTHONOPATH':os.environ['PYTHONPATH'] if 'PYTHONPATH' in os.environ else '',
-                                       'PATH':os.environ['PATH'] if 'PATH' in os.environ else '',
-                                       #'DASK_DISTRIBUTED__COMM__TIMEOUTS__TCP':"30s",
-                                       #'DASK_DISTRIBUTED__COMM__TIMEOUTS__CONNECT':"30s",
-                                       #'DASK_DISTRIBUTED__SCHEDULER__WORK_STEALING':'false',
-                                       #'DASK_DISTRIBUTED__ADMIN__TICK__LIMIT':"30s",
-                                       #'DASK_DISTRIBUTED__ADMIN__TICK__INTERVAL':"2s",
-                                       #'DASK_DISTRIBUTED__DEPLOY__LOST_WORKER_TIMEOUT':"30s"
-                                   },
-                                  direct_to_workers=True,
-                                  silence_logs=20 if verbose else 40,  # logging.WARN=30 or logging.ERROR=40 or logging.INFO=20
+                                  env=env,
+                                  direct_to_workers=True, 
+                                  #memory_limit='auto',
+                                  #silence_logs=20 if verbose else 40, 
                                   local_directory=tempfile.mkdtemp())
 
         self._num_gpus = num_gpus
@@ -167,19 +175,19 @@ class Batch(Checkpoint):
     >>> b = vipy.batch.Batch(v, n_processes=16)
     >>> b.map(lambda v,k: v[k], args=[(k,) for k in range(0, len(v))])  # paralle interpolation
 
-    >>> d = vipy.dataset.kinetics.Kinetics700('/path/to/kinetics').download().trainset()
+    >>> d = vipy.data.kinetics.Kinetics700('/path/to/kinetics').download().trainset()
     >>> b = vipy.batch.Batch(d, n_processes=32)
     >>> b.map(lambda v: v.download().save())  # will download and clip dataset in parallel
 
     >>> b.result()  # retrieve results after a sequence of map or filter chains
 
-    Parameters:
-      -strict=False: if distributed processing fails, return None for that element and print the exception rather than raise
-      -as_completed=True:  Return the objects to the scheduler as they complete, this can introduce instabilities for large complex objects, use with caution
+    Args:
+        strict: [bool] if distributed processing fails, return None for that element and print the exception rather than raise
+        as_completed: [bool] Return the objects to the scheduler as they complete, this can introduce instabilities for large complex objects, use with caution
 
     """    
              
-    def __init__(self, objlist, strict=True, as_completed=False, checkpoint=False, checkpointdir=None, checkpointfrac=0.1, warnme=True, minscatter=None):
+    def __init__(self, objlist, strict=False, as_completed=False, checkpoint=False, checkpointdir=None, checkpointfrac=0.1, warnme=False, minscatter=None):
         """Create a batch of homogeneous vipy.image objects from an iterable that can be operated on with a single parallel function call
         """
         assert isinstance(objlist, list), "Input must be a list"
@@ -194,8 +202,9 @@ class Batch(Checkpoint):
         if checkpoint:
             as_completed = True  # force self._as_completed=True
 
+        # Move this into map and disable using localmap
         if vipy.globals.dask() is None and warnme:
-            print('[vipy.batch.Batch]: vipy.batch.Batch() is not set to use parallelism.  This is set using:\n    >>> vipy.globals.parallel(n) for multi-processing with n processes\n    >>> vipy.globals.parallel(pct=0.8) for multiprocessing that uses a percentage of the current system resources\n    >>> vipy.globals.dask(address="SCHEDULER:PORT") which connects to a Dask distributed scheduler.\n    >>> vipy.globals.noparallel() to completely disable all parallelism.')
+            print('[vipy.batch.Batch]: vipy.batch.Batch() is not set to use parallelism.  This is set using:\n    >>> with vipy.globals.parallel(n) for multi-processing with n processes\n    >>> vipy.globals.parallel(pct=0.8) for multiprocessing that uses a percentage of the current system resources\n    >>> vipy.globals.dask(address="SCHEDULER:PORT") which connects to a Dask distributed scheduler.\n    >>> vipy.globals.noparallel() to completely disable all parallelism.')
 
         # FIXME: this needs to go into Dask()
         #self._ngpu = ngpu
@@ -310,8 +319,14 @@ class Batch(Checkpoint):
         >>> imb = vipy.image.Batch(iml) 
         >>> imb.map(lambda im: im.rgb())  
 
-        The lambda function f_lambda must not include closures.  If it does, construct the batch with tuples (obj,prms) or with default parameter capture:
-        >>> f = lambda x, prm1=1, prm2=2: x+prm1+prm2
+        The lambda function f_lambda should not include closures.  If it does, construct the lambda with default parameter capture:
+
+        >>> f = lambda x, prm1=42: x+prm1
+
+        instead of:
+
+        >>> prm1 = 42
+        >>> f = lambda x: x+prm1
 
         """
         c = self._client()
@@ -321,7 +336,7 @@ class Batch(Checkpoint):
             self._objlist = [f_lambda_ordered(o) for o in self._objlist]  # no parallelism
         else:
             f_lambda_ordered = lambda x,f=f_lambda: (x[0], f(x[1]))            
-            objlist = c.scatter(self._objlist) if (self._minscatter is not None and len(self._objlist) > self._minscatter) else self._objlist
+            objlist = c.scatter(self._objlist) if (self._minscatter is not None and len(self._objlist) >= self._minscatter) else self._objlist
             self._objlist = self._wait(c.map(f_lambda_ordered, objlist))
         return self
 
@@ -361,7 +376,7 @@ class Batch(Checkpoint):
             self._objlist = [f_lambda_ordered(obj, o) for o in self._objlist]  # no parallelism
         else:
             objdist = c.scatter(obj, broadcast=True)        
-            objlist = c.scatter(self._objlist) if len(self._objlist) > self._minscatter else self._objlist
+            objlist = c.scatter(self._objlist) if (self._minscatter is not None and len(self._objlist) >= self._minscatter) else self._objlist
             self._objlist = self._wait([c.submit(f_lambda_ordered, objdist, im) for im in objlist])
         return self
 
