@@ -71,7 +71,18 @@ def similarity_transform(txy=(0,0), r=0, s=1):
 
 
 def affine_transform(txy=(0,0), r=0, sx=1, sy=1, kx=0, ky=0):
-    """Compose and return a 3x3 affine transformation for translation txy=(0,0), rotation r (radians), scalex=sx, scaley=sy, shearx=kx, sheary=ky"""
+    """Compose and return a 3x3 affine transformation for translation txy=(0,0), rotation r (radians), scalex=sx, scaley=sy, shearx=kx, sheary=ky.
+    
+    Usage:
+    
+    >>> A = vipy.geometry.affine_transform(r=np.pi/4)
+    >>> vipy.image.Image(array=vipy.geometry.imtransform(im.array(), A), colorspace='float')
+    
+    Equivalently:
+
+    >>> im = vipy.image.RandomImage().affine_transform(A)    
+    
+    """
     assert istuple(txy) and len(txy) == 2 and isnumber(r) and isnumber(sx) and isnumber(sy) and isnumber(kx) and isnumber(ky), "Invalid input"
     R = np.mat([[np.cos(r), -np.sin(r), 0], [np.sin(r), np.cos(r), 0], [0,0,1]])
     S = np.mat([[sx,0,0], [0, sy, 0], [0,0,1]])
@@ -95,11 +106,12 @@ def random_affine_transform(txy=((0,1),(0,1)), r=(0,1), sx=(0.1,1), sy=(0.1,1),
 def imtransform(img, A):
     """Transform an numpy array image (MxNx3) following the affine or similiarity transformation A"""
     assert isnumpy(img) and isnumpy(A), "invalid input"
-    try_import(cv2, 'opencv-python'); import cv2
-    if A.shape == (2,3):
-        return cv2.warpAffine(img, A, (img.shape[1], img.shape[0]))
+    try_import('cv2', 'opencv-python'); import cv2
+    if A.shape == (3,3):
+        return cv2.warpPerspective(img, A, (img.shape[1], img.shape[0]))        
     else:
-        return cv2.warpPerspective(img, A, (img.shape[1], img.shape[0]))
+        return cv2.warpAffine(img, A, (img.shape[1], img.shape[0]))        
+
 
 
 def normalize(x, eps=1E-16):
@@ -202,8 +214,8 @@ class BoundingBox(object):
         return BoundingBox(xmin=self._xmin, xmax=self._xmax, ymin=self._ymin, ymax=self._ymax)
 
     def __eq__(self, other):
-        """Bounding box equality"""
-        return isinstance(other, BoundingBox) and self.xywh() == other.xywh()
+        """Bounding box equality (integer resolution of corners)"""
+        return isinstance(other, BoundingBox) and self.clone().int().xywh() == other.clone().int().xywh()
 
     def __neq__(self, other):
         """Bounding box non-equality"""
@@ -349,6 +361,7 @@ class BoundingBox(object):
         return self
 
     def to_origin(self):
+        """Translate the bounding box so that (xmin, ymin) = (0,0)"""
         return self.translate(-self.xmin(), -self.ymin())
     
     def set_origin(self, other):
@@ -619,7 +632,7 @@ class BoundingBox(object):
         
     def ispointinside(self, p):
         """Is the 2D point p=(x,y) inside this boundingbox, or is the p=boundingbox() inside this bounding box?"""
-        assert len(p) == 2, "Invalid 2D point=(x,y) input"""
+        assert len(p) == 2, "Invalid 2D point=(x,y) input"
         return (p[0] >= self._xmin) and (p[1] >= self._ymin) and (p[0] <= self._xmax) and (p[1] <= self._ymax)
 
     def dilate(self, scale=1):
