@@ -40,7 +40,7 @@ class Detection(BoundingBox):
         self.attributes = {} if attributes is None else attributes.copy()  # shallow copy
 
     @classmethod
-    def cast(cls, d, flush=False, category=None):
+    def cast(cls, d, flush=False, category=None, shortlabel=None):
         assert isinstance(d, BoundingBox)
         if d.__class__ != Detection or flush:
             d.__class__ = Detection
@@ -52,6 +52,8 @@ class Detection(BoundingBox):
             d.attributes = {} if flush or not hasattr(d, 'attributes') else d.attributes
         if category is not None:
             d._label = category  # when casting BoundingBox to Detection, extra args are necessary
+        if shortlabel is not None:
+            d._shortlabel = shortlabel  # when casting BoundingBox to Detection, extra args are necessary
         return d
         
     @classmethod
@@ -77,9 +79,9 @@ class Detection(BoundingBox):
         return str('<vipy.object.detection: %s>' % (', '.join(strlist)))
 
     def __eq__(self, other):
-        """Detection equality when bounding boxes and categories are equivalent"""
-        return ((isinstance(other, Detection) and self.xywh() == other.xywh() and self.category() == other.category()) or
-                isinstance(other, BoundingBox) and self.xywh() == other.xywh())
+        """Detection equality when bounding boxes (integer resolution) and categories are equivalent"""
+        return ((isinstance(other, Detection) and self.clone().int().xywh() == other.clone().int().xywh() and self.category() == other.category()) or
+                isinstance(other, BoundingBox) and self.clone().int().xywh() == other.clone().int().xywh())
 
     def __str__(self):
         return self.__repr__()
@@ -182,14 +184,14 @@ class Track(object):
 
     """
 
-    def __init__(self, keyframes, boxes, category=None, label=None, framerate=None, interpolation='linear', boundary='strict', shortlabel=None, attributes=None, trackid=None):
+    def __init__(self, keyframes, boxes, category=None, label=None, framerate=None, interpolation='linear', boundary='strict', shortlabel=None, attributes=None, trackid=None, filterbox=False):
 
         keyframes = tolist(keyframes)
         boxes = tolist(boxes)        
         assert isinstance(keyframes, tuple) or isinstance(keyframes, list), "Keyframes are required and must be tuple or list"
         assert isinstance(boxes, tuple) or isinstance(boxes, list), "Keyframe boundingboxes are required and must be tuple or list"
         assert all([isinstance(bb, BoundingBox) for bb in boxes]), "Keyframe bounding boxes must be vipy.geometry.BoundingBox objects"
-        assert all([bb.isvalid() for bb in boxes]), "All keyframe bounding boxes must be valid"        
+        assert filterbox or all([bb.isvalid() for bb in boxes]), "All keyframe bounding boxes must be valid"        
         assert not (label is not None and category is not None), "Constructor requires either label or category kwargs, not both"                
         assert len(keyframes) == len(boxes), "Boxes and keyframes must be the same length, there must be a one to one mapping of frames to boxes"
         assert boundary in set(['extend', 'strict']), "Invalid interpolation boundary - Must be ['extend', 'strict']"
@@ -211,6 +213,15 @@ class Track(object):
             self._keyframes = list(keyframes)
             self._keyboxes = list(boxes)
 
+        # Filter boxes:  remove invalid boxes and keyframes
+        if filterbox and len(keyframes) > 0 and len(boxes) > 0:
+            kfbb = [(f,bb) for (f,bb) in zip(keyframes, boxes) if bb.isvalid()]
+            (keyframes, boxes) = zip(*kfbb) if len(kfbb)>0 else ([],[])
+            self._keyframes = list(keyframes)
+            self._keyboxes = list(boxes)
+            if len(self) == 0:
+                warnings.warn('vipy.object.Track - filtering invalid boxes with filterbox=True resulted in zero length track for track ID %s' % str(self.id()))            
+            
     @classmethod
     def from_json(cls, s):
         d = json.loads(s) if not isinstance(s, dict) else s
@@ -267,6 +278,14 @@ class Track(object):
         return C[0] if len(C) == 1 else (float(np.mean(C)) if len(C) > 0 else 0)
         
     def isdegenerate(self):
+        """Is the track degenerate?  
+        
+        A degenerate track has:
+            - Unequal length keyboxes and keyframes
+            - length zero track
+            - Non increasing keyframes
+            - Invalid keyboxes
+        """
         return not (len(self.keyboxes()) == len(self.keyframes()) and
                     (len(self) == 0 or all([bb.isvalid() for bb in self.keyboxes()])) and
                     sorted(self.keyframes()) == list(self.keyframes()))
@@ -277,9 +296,11 @@ class Track(object):
 
     
     def add(self, keyframe, bbox, strict=True):
-        """Add a new keyframe and associated box to track, preserve sorted order of keyframes. 
+        """Add a new keyframe and associated box to track, preserve sorted order of keyframes.  If keyframe is already in track, throw an exception.  In this case use update() instead
 
            -strict [bool]:  If box is degenerate, throw an exception if strict=True, otherwise just don't add it
+        
+        .. note::  The BoundingBox is added by reference.  If you want to this to be a copy, pass in bbox.clone()
         """
         assert isinstance(bbox, BoundingBox), "Invalid input - Box must be vipy.geometry.BoundingBox()"
         assert strict is False or bbox.isvalid(), "Invalid input - Box must be non-degenerate"
@@ -287,7 +308,7 @@ class Track(object):
         if not bbox.isvalid():            
             return self  # just don't add it 
         self._keyframes.append(int(keyframe))
-        self._keyboxes.append(bbox)
+        self._keyboxes.append(bbox)  # not cloned()
         if len(self._keyframes) > 1 and keyframe < self._keyframes[-2]:
             # Preserve sorted order if inserting into the middle somewhere
             (self._keyframes, self._keyboxes) = zip(*sorted([(f,bb) for (f,bb) in zip(self._keyframes, self._keyboxes)], key=lambda x: x[0]))        
@@ -341,13 +362,39 @@ class Track(object):
         return BoundingBox(ulbr=np.mean([bb.ulbr() for bb in self.keyboxes()], axis=0)) if len(self.keyboxes()) > 0 else None 
     
     def shapevariance(self):
-        """Return the variance (width, height) of the box shape during the track or None if the track is degenerate.  This is useful for filtering spurious tracks where the aspect ratio changes rapidly and randomly"""
+        """Return the variance (width, height) of the box shape relative to `vipy.object.Track.meanbox` during the track or None if the track is degenerate.  
+
+        This is useful for filtering spurious tracks where the aspect ratio changes rapidly and randomly
+
+        Returns:
+            (width_variance, height_variance) of the box shape during the track (or None)
+        """
         m = self.meanshape()
         return (float(np.mean([(bb.width() - m[0])**2 for bb in self.keyboxes()])), 
                 float(np.mean([(bb.height() - m[1])**2 for bb in self.keyboxes()]))) if m is not None else None
 
+    def _set_framerate(self, fps):
+        """Override framerate conversion and just set the framerate attribute.  
+
+        .. warning::  This should really only be set by the user in the constructor and is included here as an admin override for some legacy JSON that did not contain framerates.  Use with caution!
+        """
+        self._framerate = fps
+        return self
+
     def framerate(self, fps=None, speed=None):
-        """Resample keyframes from known original framerate set by constructor to be new framerate fps"""
+        """Resample keyframes from known original framerate set by constructor to be new framerate fps.
+
+        Args:
+            fps: [float]  The new frame rate in frames per second
+            speed: [float]  An optional speed factor which will multiply the current framerate by this factor (e.g. speed=2 --> fps=self.framerate()*2)
+
+        Returns:
+            This track object with the keyframes resampled to the new framerate
+
+        """
+        if fps is None and speed is None:
+            return self._framerate
+        
         assert self._framerate is not None, "Framerate conversion requires that the framerate is known for current keyframes.  This must be provided to the vipy.object.Track() constructor."
         assert fps is not None or speed is not None, "Invalid input"
         assert not (fps is not None and speed is not None), "Invalid input"
@@ -359,25 +406,36 @@ class Track(object):
         return self
         
     def startframe(self):
+        """Return the startframe of the track or None if there are no keyframes.  
+        
+        The frame index is relative to the framerate set in the constructor.
+
+        """        
         return self._keyframes[0] if len(self._keyframes)>0 else None  # assumes sorted order
 
     def endframe(self):
+        """Return the endframe of the track or None if there are no keyframes.
+
+        The frame index is relative to the framerate set in the constructor.
+        """
         return self._keyframes[-1] if len(self._keyframes)>0 else None  # assumes sorted order
 
     def linear_interpolation(self, f, id=True):
         """Linear bounding box interpolation at frame=k given observed boxes (x,y,w,h) at keyframes.  
-        This returns a vipy.object.Detection() which is the interpolation of the Track() at frame k
-        If self._boundary='extend', then boxes are repeated if the interpolation is outside the keyframes
-        If self._boundary='strict', then interpolation returns None if the interpolation is outside the keyframes
+
+        This returns a `vipy.object.Detection` which is the interpolation of the `vipy.object.Track` at frame k
+
+        - If self._boundary='extend', then boxes are repeated if the interpolation is outside the keyframes
+        - If self._boundary='strict', then interpolation returns None if the interpolation is outside the keyframes
         
-          - The returned object is not cloned when possible for speed purposes, be careful when modifying this object.  clone() if necessary
+        .. note::  The returned object is not cloned when possible for speed purposes, be careful when modifying this object.  clone() if necessary
 
         """
         assert len(self._keyboxes) > 0, "Degenerate object for interpolation"   # not self.isempty()
         if len(self._keyboxes) == 1:
-            return Detection.cast(self._keyboxes[0].clone(), category=self.category()).noattributes().setattribute('trackid', self.id()) if (self._boundary == 'extend' or self.during(f)) else None
+            return Detection.cast(self._keyboxes[0].clone(), category=self.category(), shortlabel=self.shortlabel()).noattributes().setattribute('trackid', self.id()) if (self._boundary == 'extend' or self.during(f)) else None
         if f in reversed(self._keyframes):            
-            return Detection.cast(self._keyboxes[self._keyframes.index(f)], category=self.category()).noattributes().setattribute('trackid', self.id())
+            return Detection.cast(self._keyboxes[self._keyframes.index(f)], category=self.category(), shortlabel=self.shortlabel()).noattributes().setattribute('trackid', self.id())
 
         kf = self._keyframes
         ft = min(max(f, kf[0]), kf[-1])  # truncated frame index
@@ -398,9 +456,12 @@ class Track(object):
         return d if self._boundary == 'extend' or self.during(f) else None
 
     def category(self, label=None, shortlabel=True):
+        """Set the track category to label, and update thte shortlabel also.  Updates all keyboxes"""
         if label is not None:
             self._label = str(label)  # coerce to string
             self._shortlabel = str(label) if shortlabel else self._shortlabel  # coerce to string
+            self.boxmap(lambda bb: bb.shortlabel(self._shortlabel) if shortlabel and isinstance(bb, Detection) else bb)
+            self.boxmap(lambda bb: bb.category(self._label) if isinstance(bb, Detection) else bb)
             return self
         else:
             return self._label
@@ -410,42 +471,102 @@ class Track(object):
         return self.category(label, shortlabel=True)
         
     def shortlabel(self, label=None):
-        """A optional shorter label string to show as a caption in visualizations"""                
+        """A optional shorter label string to show as a caption in visualizations.  Updates all keyboxes"""                
         if label is not None:
             self._shortlabel = str(label)  # coerce to string
+            self.boxmap(lambda bb: bb.shortlabel(self._shortlabel) if isinstance(bb, Detection) else bb)
             return self
         else:
             return self._shortlabel
 
     def during(self, k_start, k_end=None):
-        """Is frame during the time interval (startframe, endframe) inclusive?"""        
+        """Does the track contain a keyframe during the time interval (startframe, endframe) inclusive?"""        
         k_end = k_start+1 if k_end is None else k_end
         (startframe, endframe) = (self.startframe(), self.endframe())
         return len(self)>0 and ((k_start >= startframe and k_start <= endframe) or (k_end >= startframe and k_end <= endframe) or (k_start <= startframe and k_end >= endframe))
         
     def during_interval(self, k_start, k_end):
+        """Does the track contain a keyframe during the inclusive frame interval (startframe, endframe)?
+
+        .. note:: The start and end frames are inclusive
+        """
         return self.during(k_start, k_end)
 
+    def within(self, starframe, endframe):
+        """Is the track within the frame range (startframe, endframe)?"""
+        return self.startframe() >= startframe and self.endframe() <= endframe
+    
     def offset(self, dt=0, dx=0, dy=0):
+        """Apply a temporal shift of dt frames, and a spatial shift of (dx, dy) pixels.
+        
+        Args:
+            dt: [int] frame offset
+            dx: [float] horizontal spatial offset 
+            dy: [float] vertical spatial offset 
+
+        Returns:
+            This box updated in place
+        """
         self._keyboxes = [bb.offset(dx, dy) for bb in self._keyboxes]
         self._keyframes = [int(f+dt) for f in self._keyframes]
         return self
 
+    def uncrop(self, bb, s=1):
+        """Apply a transformation to the track that will undo a crop of a bounding box with an optional scale factor.
+
+        A typical operation is as follows.  A video is cropped and zommed in order to run a detector on a region of interest.  However, we want to align the resulting tracks on the original video before the crop and zoom.  
+
+        Args:
+            bb: [`vipy.geometry.BoundingBox`].  A bounding box which was used to crop this track
+            s: [float]  A scale factor applied after the bounding box crop
+
+        Returns:
+            This track after undoing the scale and crop 
+        """
+        assert isinstance(bb, BoundingBox)
+        return self.rescale(1/s).offset(dt=0, dx=bb.xmin(), dy=bb.ymin())
+
     def frameoffset(self, dx, dy):
-        """Offset boxes by (dx,dy) in each frame"""
+        """Offset boxes by (dx,dy) in each frame.
+        
+        This is used to apply a different offset for each frame.  To apply one offset to all frames, use `vipy.object.Track.offset`.
+        Args:
+            dx: [list]  This should be a list of frame offsets at each keyframe the same length as the number of keyboxes
+            dy: [list]  This should be a list of frame offsets at each keyframe the same length as the number of keyboxes
+
+        Returns:
+            This track updated in place
+        """
+        assert isinstance(dx, list) or isinstance(dx, tuple)
+        assert isinstance(dy, list) or isinstance(dy, tuple)
         assert len(self.keyboxes()) == len(dx) and len(self.keyboxes()) == len(dy)
         self._keyboxes = [bb.offset(dx=x, dy=y) for (bb, (x, y)) in zip(self._keyboxes, zip(dx, dy))]
         return self
 
     def truncate(self, startframe=None, endframe=None):
-        """Truncate a track so that any keyframes less than startframe or greater than endframe are removed"""
+        """Truncate a track so that any keyframes less than startframe or greater than endframe (inclusive) are removed.  Interpolate keyboxes at (startframe, endframe) endpoints.
+
+        Args:
+            startframe: [int] The startframe of the truncation relative to the track framerate.  All keyframes less than or equal to startframe are included.  If the keyframe does not exist at startframe, one is interpolated and added.
+            endframe: [int] The endframe of the truncation relative to the track framerate.  All keyframes greater than or equal to the endframe are included.  If the keyfrmae does not exist at endframe, one is interpolated and added.
+
+        Returns:
+            This track such that all keyboxes <= startframe or >= endframe are removed.
+
+        .. note::  The startframe and endframe for truncation are inclusive.  
+        """
+        if startframe is not None and startframe not in self._keyframes and self[startframe] is not None:
+            self.add(startframe, self[startframe].clone())  # interpolated boundary condition
+        if endframe is not None and endframe not in self._keyframes and self[endframe] is not None:
+            self.add(endframe, self[endframe].clone())  # intepolated boundary condition
         kfkb = [(kf,kb) for (kf,kb) in zip(self._keyframes, self._keyboxes) if ((startframe is None or kf >= startframe) and (endframe is None or kf <= endframe))]
         (self._keyframes, self._keyboxes) = zip(*kfkb) if len(kfkb) > 0 else ([], [])
         return self
         
     def rescale(self, s):
         """Rescale track boxes by scale factor s"""
-        self._keyboxes = [bb.rescale(s) for bb in self._keyboxes]
+        if s != 1.0:
+            self._keyboxes = [bb.rescale(s) for bb in self._keyboxes]
         return self
 
     def scale(self, s):
@@ -467,6 +588,11 @@ class Track(object):
         self._keyboxes = [bb.dilate(s) for bb in self._keyboxes]
         return self
 
+    def maxsquare(self):
+        """Set all of the track boxes to maxsquare"""
+        self._keyboxes = [bb.maxsquare() for bb in self._keyboxes]
+        return self
+    
     def rot90cw(self, H, W):
         """Rotate an image with (H,W)=shape 90 degrees clockwise and update all boxes to be consistent"""
         self._keyboxes = [bb.rot90cw(H, W) for bb in self._keyboxes]
@@ -494,18 +620,37 @@ class Track(object):
             self._id = newid
             return self
 
-    def clone(self, startframe=None, endframe=None):
+    def clone(self, startframe=None, endframe=None, rekey=False):
         #return copy.deepcopy(self)  
-        return Track.from_json(self.json(encode=False)) if (startframe is None and endframe is None) else self.clone_during(startframe, endframe)  # 2x faster than deepcopy
-
+        t = Track.from_json(self.json(encode=False)) if (startframe is None and endframe is None) else self.clone_during(startframe, endframe)  # 2x faster than deepcopy
+        if rekey:
+            global DETECTION_GUID; t.id(newid=hex(int(DETECTION_GUID))[2:]);  DETECTION_GUID = DETECTION_GUID + 1;  # faster, increment package level UUID4 initialized GUID
+        return t
+    
     def clone_during(self, startframe, endframe):
-        #return copy.deepcopy(self)
+        """Clone a track during a specific interval (startframe, endframe) relative to the framerate of the track.
+
+        - This is useful for copying a small segment of a long track without the expense of copying the whole track.  
+        - All keyframes and keyboxes not in (startframe, endframe) are not copied.
+        - Boundary keyframes are copied to enable proper interpolation.        
+        """
+        # Update (startframe,endframe) to be the keyframes just before startframe and the keyframe just after endframe so that interpolation will work correctly
+        (startframe, endframe) = (([kf for kf in self._keyframes if kf <= startframe][-1]) if self.during(startframe, startframe) else startframe,
+                                  ([kf for kf in self._keyframes if kf >= endframe][0]) if self.during(endframe, endframe) else endframe)
         kfkb = [(kf,kb.clone()) for (kf,kb) in zip(self._keyframes, self._keyboxes) if ((startframe is None or kf >= startframe) and (endframe is None or kf <= endframe))]
         (kf, kb) = zip(*kfkb) if len(kfkb) > 0 else ([], [])        
         return Track(keyframes=kf, boxes=kb, category=self._label, framerate=self._framerate, interpolation=self._interpolation, boundary=self._boundary, shortlabel=self._shortlabel, attributes=self.attributes, trackid=self._id)
     
     def boundingbox(self, startframe=None, endframe=None):
-        """The bounding box of a track is the smallest spatial box that contains all of the detections within startframe and endframe, or None if there are no detections"""
+        """The bounding box of a track is the smallest spatial box that contains all of the BoundingBoxes of the track  within startframe and endframe, or None if there are no detections.
+        
+        Args:
+            startframe: [int] the startframe of the track to compute the bounding box.
+            endframe: [int] the endframe of the track to compute the bounding box.
+        
+        Returns:
+            `vipy.geometry.BoundingBox` which is the smallest box that contains all boxes of the track from (startframe, endframe)
+        """
         t = self.clone() if (startframe is None and endframe is None) else self.clone().truncate(startframe, endframe)
         d = t._keyboxes[0].clone() if len(t._keyboxes) >= 1 else None
         return d.union([bb for (k,bb) in zip(t._keyframes[1:], t._keyboxes[1:]) if t.during(k)]) if (d is not None and len(t._keyboxes) >= 2) else d
@@ -593,6 +738,13 @@ class Track(object):
         startframe = max(self.startframe(), other.startframe())
         endframe = min(self.endframe(), other.endframe())   # inclusive
         return float(np.mean([self[min(k,endframe)].iou(other[min(k,endframe)]) for k in range(startframe, endframe, dt)]) if endframe > startframe else 0.0)
+
+    def segmentcover(self, other, dt=5):
+        """Compute the mean spatial cover between two tracks at the overlapping segment, sampling by dt.  Useful for track continuation for densely overlapping tracks"""
+        assert isinstance(other, Track), "invalid input - Must be vipy.object.Track()"
+        startframe = max(self.startframe(), other.startframe())
+        endframe = min(self.endframe(), other.endframe())   # inclusive
+        return float(np.mean([self[min(k,endframe)].maxcover(other[min(k,endframe)]) for k in range(startframe, endframe, dt)]) if endframe > startframe else 0.0)
         
     def rankiou(self, other, rank, dt=1):
         """Compute the mean spatial IoU between two tracks per frame in the range (self.startframe(), self.endframe()) using only the top-k (rank) frame overlaps
@@ -619,7 +771,8 @@ class Track(object):
         """
         assert percentile > 0 and percentile <= 1
         assert isinstance(other, Track), "invalid input - Must be vipy.object.Track()"
-
+        assert self.framerate() == other.framerate()
+        
         startframe = max(self.startframe(), other.startframe())
         endframe = min(self.endframe(), other.endframe())
         segmentlen = endframe - startframe
@@ -630,7 +783,8 @@ class Track(object):
         """percentiliou on the overlapping segment with other"""
         assert percentile > 0 and percentile <= 1
         assert isinstance(other, Track), "invalid input - Must be vipy.object.Track()"
-
+        assert self.framerate() == other.framerate()
+        
         startframe = max(self.startframe(), other.startframe())
         endframe = min(self.endframe(), other.endframe())
         segmentlen = endframe - startframe
@@ -644,7 +798,8 @@ class Track(object):
         """percentile cover on the overlapping segment with other"""
         assert percentile > 0 and percentile <= 1
         assert isinstance(other, Track), "invalid input - Must be vipy.object.Track()"
-
+        assert self.framerate() == other.framerate()
+        
         startframe = max(self.startframe(), other.startframe())
         endframe = min(self.endframe(), other.endframe())
         segmentlen = endframe - startframe
@@ -661,7 +816,6 @@ class Track(object):
              - average [bool]:  average framewise interpolated boxes at overlapping keyframes
              - replace [bool]:  replace the box with other if other and self overlap at a keyframe
              - keep [bool]:  keep the box from self (discard other) at a keyframe
-
         """
         assert isinstance(other, Track), "Invalid input - must be vipy.object.Track()"
         assert other.category() == self.category(), "Category mismatch"
@@ -789,7 +943,19 @@ class Track(object):
         return float(np.arctan2(v[1], v[0])) if self.speed(f, dt) > minspeed else None  # atan2(y,x)
 
     def bearing_change(self, f1=None, f2=None, dt=30, minspeed=1, samples=None):
-        """The bearing change of a track from frame f1 (or start) and frame f2 (or end) is the relative angle of the velocity vectors in radians [-pi,pi]"""
+        """The bearing change of a track from frame f1 (or start) and frame f2 (or end) is the relative angle of the velocity vectors in radians [-pi,pi].
+        
+        Args:
+            f1: [int] the start frame for computing the bearing change.  If None, then use self.startframe()
+            f2: [int] the end frame for computing the bearing change.  if None, then use self.endframe()
+            dt: [int] The number of frames between computations of the velocity vector for bearing
+            minspeed: [float] The minimum speed in frames per second used to threshold bearing computations if there is no motion
+            samples: [int] The number of samples to average for computing the bearing change
+        
+        Returns:
+            The floating point bearing change in radians in [-pi, pi] from (f1,f2) where bearing is computed at samples=n points, and each bearing is computed with a velocity stride of dt frames.
+
+        """
         dt = min(dt, len(self))
         (sf, ef) = (f1 if f1 is not None else self.startframe(), f2 if f2 is not None else self.endframe())
         df = 1 if samples is None else int(np.floor((ef-sf)/samples))
@@ -799,10 +965,14 @@ class Track(object):
         return float(dr if np.abs(dr)<=np.pi else ((2*np.pi - dr) if (dr > np.pi) else (2*np.pi + dr)))
 
     def acceleration(self, f, dt=30):
-        """Return the (x,y) track acceleration magnitude at frame f computed using central finite differences of velocity"""
+        """Return the (x,y) track acceleration magnitude at frame f computed using central finite differences of velocity.
+        
+        Returns:
+            acceleration in (pixels / seconds^2) using velocity computed at (f-2*dt, f-dt), (f+dt, f+2*dt)
+        """
         (u, v) = (self.shape_invariant_velocity(f-dt, dt), self.shape_invariant_velocity(f+2*dt, dt))  # ((f-2*dt, (f-dt)), (f+dt, f+2*dt))
         (ax, ay) = ((v[0] - u[0])/float(2*dt), (v[1] - u[1])/float(2*dt))
-        return float(np.sqrt(ax**2 + ay**2))  # acceleration magnitude in pixels
+        return float(np.sqrt(ax**2 + ay**2))  # acceleration magnitude in pixels    
         
     def velocity(self, f, dt=30):
         """Return the (x,y) track velocity at frame f in units of pixels per frame computed by mean finite difference of the box centroid"""
@@ -814,7 +984,8 @@ class Track(object):
     
     def boxmap(self, f):
         """Apply the lambda function to each keybox"""
-        self._keyboxes = [f(bb) for bb in self._keyboxes]
+        assert callable(f)
+        self._keyboxes = [f(bb) for bb in self._keyboxes]        
         return self
 
     def shape_invariant_velocity(self, f, dt=30):
@@ -866,7 +1037,20 @@ class Track(object):
 
     
 def non_maximum_suppression(detlist, conf, iou, bycategory=False, cover=None, gridsize=(6,9)):
-    """Compute greedy non-maximum suppression of a list of vipy.object.Detection() based on spatial IOU threshold (iou) and cover threhsold (cover) sorted by confidence (conf)"""
+    """Compute greedy non-maximum suppression of a list of vipy.object.Detection() based on spatial IOU threshold (iou) and cover threhsold (cover) sorted by confidence (conf).
+
+    Args:
+        detlist: [list `vipy.object.Detection`]
+        conf: [float] minimum confidence for non-maximum suppression
+        iou: [float] minimum iou for non-maximum suporession
+        bycategory: [bool] NMS only within the same category 
+        cover: [float, None] A minimum cover for NMS (stricter than iou)
+        gridsize: [tuple, (rows, cols)] An optional grid for fast intersection lookups 
+
+    Returns:
+        List of `vipy.object.Detection` non-maximum suppressed, sorted by increasing confidence 
+
+    """
     assert all([isinstance(d, Detection) for d in detlist])
     assert all([d.confidence() is not None for d in detlist])
     assert conf>=0 and iou>=0 and iou<=1
@@ -900,8 +1084,14 @@ def non_maximum_suppression(detlist, conf, iou, bycategory=False, cover=None, gr
 def greedy_assignment(srclist, dstlist, miniou=0.0, bycategory=False):
     """Compute a greedy one-to-one assignment of each vipy.object.Detection() in srclist to a unique element in dstlist with the largest IoU greater than miniou, else None
     
-       returns:
-          assignlist [list]:  same length as srclist, where j=assignlist[i] is the index of the assignment such that srclist[i] == dstlist[j]
+    Args:
+        srclist: [list, `vipy.object.Detection`]
+        dstlist: [list, `vipy.object.Detection`]
+        miniou: [float, >=0, <=1] The minimum IoU for gated assignment
+        bycategory: [bool]  If true, only assign di and dj if di.category() == dj.category()
+
+    Returns:
+        assignlist: [list, int]  same length as srclist, where j=assignlist[i] is the index of the assignment such that srclist[i] -> dstlist[j]
     """
     assert all([isinstance(d, Detection) for d in srclist])
     assert all([isinstance(d, Detection) for d in dstlist])    
@@ -909,11 +1099,41 @@ def greedy_assignment(srclist, dstlist, miniou=0.0, bycategory=False):
     
     assigndict = {}
     for (k, ds) in sorted(enumerate(srclist), key=lambda x: x[1].area(), reverse=True):
-        iou = [ds.iou(d) if (j not in assigndict.values() and (bycategory is False or ds.category() == d.category())) else 0.0 for (j,d) in enumerate(dstlist)]
+        iou = [ds.iou(d) if (j not in assigndict.values() and (bycategory is False or ds.category().lower() == d.category().lower())) else 0.0 for (j,d) in enumerate(dstlist)]
         assigndict[k] = np.argmax(iou) if len(iou) > 0 and max(iou) > miniou else None
     return [assigndict[k] for k in range(0, len(srclist))]
 
+
+def greedy_track_assignment(srclist, dstlist, miniou, bycategory=True, pct=0.5):
+    """Compute a greedy one-to-ine assignment of each `vipy.object.Track` in srclist to a unique element in dstlist with the largest assignment score.
+
+    - Assignment score: `vipy.object.Track.segment_percentileiou` * `vipy.object.Track.confidence`, if maxiou() > miniou else 0
+    - Assigment order: longest to shortest src track
+
+    Args:
+        srclist: [list, `vipy.object.Track`]
+        dstlist: [list, `vipy.object.Track`]
+        miniou: [float, >=0, <=1] The minimum IoU for gated assignment
+        bycategory: [bool]  If true, only assign di and dj if di.category() == dj.category()
+        pct: [float <=1] The percentile for percentileiou
+
+    Returns:
+        assignlist: [list, int]  same length as srclist, where j=assignlist[i] is the index of the assignment such that srclist[i] -> dstlist[j]
+    """
+
+    assert all([isinstance(d, Track) for d in srclist])
+    assert all([isinstance(d, Track) for d in dstlist])    
+    assert miniou >= 0 and miniou <= 1.0
+    
+    assigndict = {}
+    for (k, ts) in sorted(enumerate(srclist), key=lambda x: len(x[1]), reverse=True):
+        assignscore = [ts.segment_percentileiou(t, pct) * t.confidence() if (j not in assigndict.values() and (bycategory is False or ts.category().lower() == t.category().lower()) and (miniou == 0 or ts.maxiou(t) > miniou)) else 0.0 for (j,t) in enumerate(dstlist)]
+        assigndict[k] = np.argmax(assignscore) if len(assignscore) > 0 and max(assignscore) > 0 else None
+    return [assigndict[k] for k in range(0, len(srclist))]
+    
+    
 def RandomDetection(W=640, H=480):
+    """Return a random `vipy.object.Detection` in the range (0 < xmin < W, 0 < ymin < H, height < 100, width < 100).  Useful for unit testing."""
     return Detection(xmin=np.random.rand()*W, ymin=np.random.rand()*H, width=np.random.rand()*100, height=np.random.rand()*100, category=str(np.random.rand()), confidence=np.random.rand())
 
 
