@@ -36,7 +36,7 @@ except ImportError:
 
 
 def bz2pkl(filename, obj=None):
-    """Read/Write compressed pickle file"""
+    """Read/Write a bz2 compressed pickle file"""
     assert filename[-8:] == '.pkl.bz2', "Invalid filename - must be '*.pkl.bz2'"
     if obj is not None:
         f = bz2.BZ2File(filename, 'wb')
@@ -51,6 +51,7 @@ def bz2pkl(filename, obj=None):
         
 
 def mergedict(d1, d2):
+    """Combine keys of two dictionaries and return a dictionary deep copy"""
     assert isinstance(d1, dict) and isinstance(d2, dict)
     d = copy.deepcopy(d1)
     d.update(d2)
@@ -61,11 +62,18 @@ def hascache():
     """Is the VIPY_CACHE environment variable set?"""
     return 'VIPY_CACHE' in os.environ
 
+def cache():
+    """If the VIPY_CACHE environment variable set, return it otherwise return tempdir()"""
+    return remkdir(os.environ['VIPY_CACHE']) if hascache() else tempdir()
 
 def tocache(filename):
-    """If the VIPY_CACHE environment variable is set, then return the filename in the cache"""
-    return os.path.join(remkdir(os.environ['VIPY_CACHE']), filetail(filename)) if hascache() else filename
+    """If the VIPY_CACHE environment variable is set, then return the filename=/path/to/file.ext in the cache as VIPY_CACHE/file.ext.  Otherwise, return the file in the system temp"""
+    return os.path.join(remkdir(os.environ['VIPY_CACHE']) if hascache() else tempdir(), filetail(filename))
 
+def seconds_to_MMSS_colon_notation(sec):
+    """Convert integer seconds into MM:SS colon format.  If sec=121, then return '02:01'. """
+    assert isinstance(sec, int) and sec <= 99*60 + 59 and sec >= 0
+    return '%02d:%02d' % (int(sec/60.0), sec % 60)
 
 def try_import(package, pipname=None, message=None):
     """Show a helpful error message for missing optional packages"""
@@ -157,7 +165,10 @@ def writejson(d, outfile):
     return outfile
 
 
-def readjson(jsonfile):
+def readjson(jsonfile, strict=True):
+    """Read jsonfile=/path/to/file.json and return the json parsed object, issue warning if jsonfile does not have .json extension and strict=True"""
+    if not isjsonfile(jsonfile) and strict:
+        warnings.warn('Attempting to read JSON file "%s" without .json extension' % jsonfile)
     with open(jsonfile) as f:
         data = json.loads(f.read())
     return data
@@ -173,20 +184,29 @@ def vipy_groupby(inset, keyfunc):
     return groupby(inset, keyfunc)
 
 
-def groupbyasdict(inset, keyfunc):
-    """Return dictionary of keys and lists from groupby on unsorted inset, where keyfunc is a lambda function on elements in inset"""
-    return {k: list(v) for (k, v) in groupby(inset, keyfunc)}
+def groupbyasdict(togroup, keyfunc):
+    """Return dictionary of keys and lists from groupby on unsorted inset, where keyfunc is a lambda function on elements in inset
+    
+    Args:
+        togroup: a list of elements to group
+        keyfunc:  a lambda function to operate on elemenets of togroup such that the value returned from the lambda is the equality key for grouping
 
-def countby(inset, keyfunc):
-    """Return dictionary of keys and group sizes for a grouping of the input list by keyfunc lambda function""" 
-    return {k:len(v) for (k,v) in groupbyasdict(inset, keyfunc).items()}
+    Returns:
+        A dictionary with unique keys returned from keyfunc, and values are lists of elements in togroup with the same key
+
+    """
+    return {k: list(v) for (k, v) in groupby(togroup, keyfunc)}
+
+def countby(inset, keyfunc=lambda x: x):
+    """Return dictionary of keys and group sizes for a grouping of the input list by keyfunc lambda function, sorted by increasing count""" 
+    return {k:v for (k,v) in sorted({k:len(v) for (k,v) in groupbyasdict(inset, keyfunc).items()}.items(), key=lambda x: x[1])}
 
 def most_frequent(inset):
     """Return the most frequent element as determined by element equality"""
     return sorted([(k,v) for (k,v) in vipy.util.countby(inset, lambda x: x).items()], key=lambda y: y[1])[-1][0]
 
 def countbyasdict(inset, keyfunc):
-    """Alias for countby"""
+    """Alias for `vipy.util.countby`"""
     return countby(inset, keyfunc)
 
 def softmax(x, temperature=1.0):
@@ -222,9 +242,9 @@ def dividelist(inlist, fractions):
     """Divide inlist into a list of lists such that the size of each sublist is the requseted fraction of the original list. 
        This operation is deterministic and generates the same division in multiple calls.
        
-       Input:
-         -inlist=list
-         -fractions=(0.1, 0.7, 0.2)   An iterable of fractions that must be non-negative and sum to one
+    Args:
+        inlist: [list]
+        fractions: [tuple] such as (0.1, 0.7, 0.2)   An iterable of fractions that must be non-negative and sum to one
     """
     assert all([f >= 0 and f <=1 for f in fractions])
     assert np.sum(fractions) == 1
@@ -237,8 +257,13 @@ def dividelist(inlist, fractions):
     return outlist
 
 def chunklist(inlist, num_chunks):
-    """Convert list into a list of lists of length num_chunks each element
-    is a list containing a sequential chunk of the original list"""
+    """Convert list into a list of lists of length num_chunks, such that each element is a list containing a sequential chunk of the original list.
+    
+    >>> (A,B,C) = vipy.util.chunklist(inlist, num_chunks=3)
+    >>> assert len(A) == len(inlist) // 3
+
+    .. note::  The last chunk will be larger for ragged chunks
+    """
     (m, n) = (num_chunks, int(np.ceil(float(len(inlist)) / float(num_chunks))))
     return [inlist[i * n:min(i * n + n, len(inlist))] for i in range(0, m)]
 
@@ -248,10 +273,7 @@ def chunklistbysize(inlist, size_per_chunk):
     containing a sequential chunk of the original list of length
     size_per_chunk"""
     assert size_per_chunk >= 1
-    num_chunks = np.maximum(int(np.ceil(len(inlist) / float(size_per_chunk))),
-                            1)
-    return chunklist(inlist, num_chunks)
-
+    return [inlist[i:i+size_per_chunk] for i in range(0,len(inlist),size_per_chunk)]
 
 def chunklistWithOverlap(inlist, size_per_chunk, overlap_per_chunk):
     """Convert list into a list of lists such that each element is a list
@@ -384,6 +406,12 @@ def jet(n, bgr=False):
     return rgb if bgr is False else np.fliplr(rgb)
 
 
+def is_email_address(email):
+    """Is the provided string an email address?"""
+    regex = r'\b[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Z|a-z]{2,}\b'
+    return re.fullmatch(regex, email) is not None
+
+
 def is_hiddenfile(filename):
     """Does the filename start with a period?"""
     return filename[0] == '.'
@@ -453,7 +481,7 @@ def _json_class_registry():
 
 def saveas(vars, outfile=None, format='dill'):
     """Save variables as a dill pickled file"""
-    outfile = temppickle() if outfile is None else outfile
+    outfile = temppickle() if outfile is None else os.path.abspath(os.path.expanduser(outfile))
 
     remkdir(filepath(outfile))
     if format in ['dill']:
@@ -502,7 +530,7 @@ def loadas(infile, format='dill'):
         raise ValueError('unknown serialization format "%s"' % format)
 
 
-def load(infile, abspath=False):
+def load(infile, abspath=True):
     """Load variables from a relocatable archive file format, either Dill Pickle or JSON.
        
        Loading is performed by attemping the following:
@@ -510,8 +538,7 @@ def load(infile, abspath=False):
        1. load the pickle or json file
        2. if abspath=true, then convert relative paths to absolute paths for object when loaded
        3. If the loaded object is a vipy object (or iterable) and the relocatable path /$PATH is present, try to repath it to the directory containing this archive (this has been deprecated)
-       4. If the resulting files are not found, throw a warning
-       5. If a large number of objects are loaded, disable garbage collection.
+       4. If the resulting first filename is not found, throw a warning
 
     """
     infile = os.path.abspath(os.path.expanduser(infile))
@@ -530,29 +557,26 @@ def load(infile, abspath=False):
 
     # Relocatable object?
     if hasattr(testobj, 'filename') and testobj.filename() is not None and '/$PATH' in testobj.filename():
+        # 07SEP21: this path is deprecated
         testobj = repath(copy.deepcopy(testobj), '/$PATH', filepath(os.path.abspath(infile)))  # attempt to rehome /$PATH/to/me.jpg -> /NEWPATH/to/me.jpg where NEWPATH=filepath(infile)
         if os.path.exists(testobj.filename()):       # file found
             obj = repath(obj, '/$PATH', filepath(os.path.abspath(infile)))      # rehome everything to the same root path as the pklfile
         else:
             warnings.warn('Loading "%s" that contains redistributable paths - Use vipy.util.distload("%s", datapath="/path/to/your/data") to rehome absolute file paths' % (infile, infile))
-    elif hasattr(testobj, 'filename') and testobj.filename() is not None and not os.path.exists(testobj.filename()):
-        if hasattr(testobj, 'hasurl') and testobj.hasurl():
-            warnings.warn('Loading archive "%s" that contains filename "%s" which does not exist - This must be downloaded from the provided url() using download()' % (infile, testobj.filename()))
-        elif not os.path.isabs(testobj.filename()):
+    elif hasattr(testobj, 'filename') and testobj.filename() is not None:
+        if not os.path.isabs(testobj.filename()):
             if not abspath:
                 warnings.warn('Loading archive "%s" with relative paths.  Changing directory to "%s".  Disable this warning with vipy.util.load(..., abspath=True).' % (infile, filepath(infile)))
                 os.chdir(filepath(infile))
             else:
-                # Absolute path?  The loaded archive will no longer be relocatable, and the videos directory cannot be moved
-                pwd = os.getcwd()  
+                # Absolute path?  The loaded archive will no longer be relocatable if you save this to a new archive, and the videos directory cannot be moved
+                pwd = os.getcwd()  # save current directory
                 os.chdir(filepath(infile))  # change to archive directory
-                objout = [o.abspath() if o.hasfilename() else o for o in tolist(obj)]  # set absolute paths
+                objout = [o.abspath() if o.filename() is not None else o for o in tolist(obj)]  # set absolute paths relative to archive directory
                 obj = objout if isinstance(obj, list) else objout[0]
-                if any([not o.hasfilename() for o in tolist(obj)]):
-                    warnings.warn('Loading "%s" that contains absolute path "%s" which does not exist' % (infile, tolist(obj)[0]))
-                os.chdir(pwd)  # set it back
-        else:
-            warnings.warn('Loading "%s" that contains absolute path "%s" which does not exist' % (infile, testobj.filename()))
+                os.chdir(pwd)  # restore current directory
+        elif not testobj.hasfilename():
+            warnings.warn('Loading "%s" that contains path (e.g. "%s") which does not exist' % (infile, testobj.filename()))
 
     # Large vipy object?  Disable garbage collection.
     #   - Python uses reference counting for the primary garbage collection mechanism, but also uses reference cycle checks to search for dependencies between objects.
@@ -562,12 +586,24 @@ def load(infile, abspath=False):
     #   - This can be re-enabled at any time by "import gc; gc.enable()"
     #   - If you use %autoreload iPython magic command, note that this will be very slow.  You should set %sutoreload 0
     #   - Alternatively, load as JSON and all attributes will be unpacked on demand and stored in a packed format that is not tracked (e.g. tuple of strings) by the reference cycle counter
-    if format != 'json' and hasattr(testobj, 'filename') and len(tolist(obj)) > 10000:
+    if hasattr(testobj, 'filename') and len(tolist(obj)) > 100000:
         # Should we warn the user?  Probably doesn't matter ...
         #warnings.warn('Loading "%s" that contains a large number of vipy objects - disabling reference cycle checks.  Re-enable at any time using "import gc; gc.enable()"' % (infile))
-        import gc; gc.disable()
+        #import gc; gc.disable()
+        pass   # do not do this without explicit user authorization
     return obj
 
+def take(inlist, k):
+    """Take k elements at random from inlist"""
+    return [inlist[i] for i in np.random.permutation(range(len(inlist)))[0:k]]
+
+def tryload(infile, abspath=False):
+    """Attempt to load a pkl file, and return the value if successful and None if not"""
+    try:
+        return load(infile, abspath=abspath)
+    except:
+        return None
+
 def canload(infile):
     """Attempt to load a pkl file, and return true if it can be successfully loaded, otherwise False"""
     try:
@@ -729,6 +765,16 @@ def isinstalled(cmd):
     return shutil.which(cmd) is not None
 
 
+def isextension(filename, ext):
+    """Does the filename end with the extension ext? 
+    
+    >>> isextension('/path/to/myfile.json', 'json') == True
+    >>> isextension('/path/to/myfile.json', '.json') == True
+    >>> isextension('/path/to/myfile.json', '.pkl') == False
+
+    """
+    return filename.endswith(ext)
+
 def ispkl(filename):
     """Is the file a pickle archive file"""
     return filename[-4:] == '.pkl' if isstring(filename) and \
@@ -740,7 +786,7 @@ def ispklfile(filename):
 
 def ishtml(filename):
     """Is the file an HTMLfile"""
-    return fileext(filename).lower() == '.html'
+    return filename.lower()[-5:] == '.html'
 
 def ispickle(filename):
     """Is the file a pickle archive file"""
@@ -759,7 +805,10 @@ def ishdf5(path):
 
 
 def filebase(filename):
-    """Return c for filename /a/b/c.ext"""
+    """Return c for filename /a/b/c.ext
+    
+    .. warning:: Will return /a/b/c.d for multidot filenames like /a/b/c.d.e (e.g. /a/b/filename.tar.gz)
+    """
     (head, tail) = os.path.split(filename)
     (base, ext) = os.path.splitext(tail)
     return base
@@ -815,7 +864,7 @@ def topath(filename, newdir):
 def filefull(f):
     """Return /a/b/c for filename /a/b/c.ext"""
     ext = fileext(f, multidot=True, withdot=True)
-    return f.replace(ext, '') if ext is not None else None
+    return f.replace(ext, '') if ext is not None else f
 
 
 def filetail(filename):
@@ -997,6 +1046,7 @@ def isurl(path):
         return False
 
 def shortuuid(n=16):
+    """Generate a short UUID with n hex digits"""
     return hashlib.sha256(uuid.uuid1().hex.encode('utf-8')).hexdigest()[0:n] 
 
 
@@ -1019,6 +1069,11 @@ def isyoutubeurl(path):
     """Is a path a youtube URL?"""
     return isurl(path) and 'youtube.com' in path
 
+def isRTSPurl(path):
+    return isurl(path) and path.startswith('rtsp://')
+
+def isRTMPurl(path):
+    return isurl(path) and (path.startswith('rtmp://') or path.startswith('rtmps://'))
 
 def checkerboard(m=8,n=256):
     """m=number of square by column, n=size of final image"""
@@ -1279,6 +1334,10 @@ def tempMP4():
     """Create a temporary MP4 file in system temp directory"""
     return tempfilename(suffix='.mp4')
 
+def tempWEBP():
+    """Create a temporary WEBP file in system temp directory"""
+    return tempfilename(suffix='.webp')
+
 
 def tmpjpg():
     """Create a temporary JPG file in /tmp"""
@@ -1468,7 +1527,7 @@ def rermdir(path):
 
 
 def premkdir(filename):
-    """create directory /path/to/subdir if not exist for outfile=/path/to/subdir/file.ext, and return filename"""
+    """pre-create directory /path/to/subdir using `vipy.util.remkdir` if it does not exist for outfile=/path/to/subdir/file.ext, and return filename"""
     remkdir(filepath(filename))
     return filename
 
@@ -1478,12 +1537,15 @@ def newbase(filename, base):
     return os.path.join(filepath(filename), '%s.%s' % (base, fileext(filename, withdot=False)))
 
 def toextension(filename, newext):
-    """Convert filenam='/path/to/myfile.ext' to /path/to/myfile.xyz, such that newext='xyz' or newext='.xyz'"""
+    """Convert filename='/path/to/myfile.ext' to /path/to/myfile.xyz, such that newext='xyz' or newext='.xyz'"""
     if '.' in newext:
         newext = newext.split('.')[-1]
     (filename, oldext) = splitextension(filename)
     return filename + '.' + str(newext)
 
+def topkl(filename):
+    """Convert filename='/path/to/myfile.ext' to /path/to/myfile.pkl"""
+    return toextension(filename, '.pkl')
 
 def splitextension(filename):
     """Given /a/b/c.ext return tuple of strings ('/a/b/c', '.ext')"""
